library("ada")

default <- rpart.control()
stump <- rpart.control(cp = -1 , maxdepth = 1 , minsplit = 0)
four <- rpart.control(cp = -1 , maxdepth = 2 , minsplit = 0)

n <- 500
p <- 10
f <- function(x, a, b, d) a * (x - b)^2 + d
set.seed(100)
x1 <- runif(n/2, 0, 4)
y1 <- f(x1, -1, 2, 1.7)+runif(n/2, -1, 1)
x2 <- runif(n/2, 2, 6)
y2 <- f(x2, 1, 4, -1.7)+runif(n/2, -1, 1)
y <- c(rep(1, n/2), rep(2, n/2))
mat <- matrix(rnorm(n * 8), ncol = 8)
dat <- data.frame(y = y, x1 = c(x1, x2), x2 = c(y1, y2), mat)
names(dat) <- c("y", paste("x", 1:10, sep = ""))
plot(dat$x1, dat$x2, pch = c(1:2)[y], col = c(1, 8)[y],
     xlab=names(dat)[2], ylab=names(dat)[3])
indtrain <- sample(1:n, 100, FALSE)
train <- dat[indtrain,]
test <- dat[-indtrain,]

default <- rpart.control()
gdis <- ada(y~., data = train, iter = 50, loss = "e", type = "discrete",
	    control = default)
gdis <- addtest(gdis, test[,-1], test[,1]))

control <- rpart.control(maxdepth = 2,cp = -1, minsplit = 0)
greal <- ada(y~., data = train, iter = 1000, type = "real", nu = 0.001,
	     bag.frac = 1, model.coef = FALSE, control = control)
ggen <- ada(y~., data = train, test.x = test[,-1], test.y = test[,1],
	    iter = 100, type = "gentle", nu = 0.1, bag.shift = TRUE,
	    control = rpart.control(cp = -1, maxdepth = 8))
glog <- ada(y~., data = train, test.x = test[,-1], test.y = test[,1],
	    iter = 50, loss = "l", type = "gentle")
greal <- ada(y~., data = train, test.x = test[,-1], test.y = test[,1],
	     iter = 50, type = "real", verbose = TRUE)

plot(gdis)
plot(gdis, FALSE, TRUE)

summary(ggen, n.iter = 64)

pred <- predict(greal, train[,-1])
table(pred)
pred <- predict(greal, train[,-1], type = "prob")
pred

test <- as.data.frame(test)
names(test) <- c("y", paste("V", 1:p, sep=""))
names(test) <- names(train)

glog <- update(glog, train[,-1], train[,1], test[,-1], test[,1], n.iter = 50)

varplot(ggen)

vip <- varplot(gdis, plot.it = FALSE, type = "scores")
round(vip, 4)

pairs(gdis, train[,-1], maxvar = 3)

pairs(gdis, train[,-1], var = 1:2,test.x = test[,-1],
      test.y = test[,1],test.only = TRUE)

n <- 12000
p <- 10
set.seed(100)
x <- matrix(rnorm(n*p), ncol=p)
y <- as.factor(c(-1, 1)[as.numeric(apply(x^2, 1, sum) > 9.34) + 1])

indtrain <- sample(1:n, 2000, FALSE)
train <- data.frame(y=y[indtrain], x[indtrain,])
test <- data.frame(y=y[-indtrain], x[-indtrain,])
control <- rpart.control(cp = -1,minsplit = 0,xval = 0,maxdepth = 1)
gdis <- ada(y~., data = train, iter = 400, bag.frac = 1, nu = 1,
	    control = control, test.x = test[,-1], test.y = test[,1])
plot(gdis, TRUE, TRUE)
summary(gdis, n.iter = 398)

data("soldat")
n <- nrow(soldat)
set.seed(100)
ind <- sample(1:n)
trainval <- ceiling(n * .5)
testval <- ceiling(n * .3)
train <- soldat[ind[1:trainval],]
test <- soldat[ind[(trainval + 1):(trainval + testval)],]
valid <- soldat[ind[(trainval + testval + 1):n],]

control <- rpart.control(cp = -1, maxdepth = 14,maxcompete = 1,xval = 0)
gen1 <- ada(y~., data = train, test.x = test[,-73], test.y = test[,73],
	    type = "gentle", control = control, iter = 70)
gen1 <- addtest(gen1, valid[,-73], valid[,73])
summary(gen1)

plot(gen1, TRUE, TRUE)
varplot(gen1)

vars <- rep(0,72)
t1 <- proc.time()
for(i in 1:20){
  rm(gen1)
  gen1 <- ada(y~., data = train, test.x = test[,-73], test.y = test[,73],
	      type = "gentle", control = control, iter = 70)
  vec1 <- varplot(gen1, plot.it = FALSE, type = "scores", max.var.show = 72)
  vars <- vars + as.numeric(vec1[order(names(vec1))]) / 20
  cat("i=", i, " time=", (proc.time()-t1)/60, "\n")
}

n <- 1200
p <- 10
K <- 3
set.seed(100)
x <- matrix(rnorm(n * p), ncol = p)
indtrain <- sample(1:n, 200, FALSE)
indtest <- setdiff(1:n, indtrain)
val <- qchisq(c(.33, .66), 10)
su <- apply(x^2, 1, sum)
Iy <- cbind(as.numeric(su <= val[1]), as.numeric( val[1] < su & su <= val[2]),
	  as.numeric(su > val[2]))
y <- apply(Iy, 1, which.max)
test <- data.frame(y = y[indtest], x[indtest,])

Fs <- list()
for(i in 1:K)
  Fs[[i]] <- ada(y~., data = data.frame(y = Iy[indtrain, i], x[indtrain,]),
		 iter = 250,test.x = test, test.y = Iy[indtest, i])$model$F[[2]]

wmx <- function(i)which.max(c(Fs[[1]][i],Fs[[2]][i],Fs[[3]][i]))
preds <- sapply(1:1000,wmx)
tab <- table(y[indtest],preds)
for(i in 1:K){
  cat("In class error rate for class ", i, ": ",
      round(1 - tab[i,i] / sum(tab[i,]), 3), "\n")
}

1 - sum(diag(tab)) / length(indtest)

library("randomForest")
train <- data.frame(y=as.factor(y[indtrain]), x[indtrain,])
set.seed(100)
grf <- randomForest(y~., train)
tab3 <- table(y[indtest], predict(grf, test))
for(i in 1:K){
  cat("In class error rate for class ", i,": ",
      round(1 - tab3[i,i] / sum(tab[i,]), 3), "\n")
}

1 - sum(diag(tab3)) / length(indtest)

